"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __classPrivateFieldGet = (this && this.__classPrivateFieldGet) || function (receiver, state, kind, f) {
    if (kind === "a" && !f) throw new TypeError("Private accessor was defined without a getter");
    if (typeof state === "function" ? receiver !== state || !f : !state.has(receiver)) throw new TypeError("Cannot read private member from an object whose class did not declare it");
    return kind === "m" ? f : kind === "a" ? f.call(receiver) : f ? f.value : state.get(receiver);
};
var _MicroRedisStartup_instances, _MicroRedisStartup_handlers, _MicroRedisStartup_pattern;
Object.defineProperty(exports, "__esModule", { value: true });
exports.MicroRedisStartup = void 0;
const micro_1 = require("@ipare/micro");
const core_1 = require("@ipare/core");
const micro_common_1 = require("@ipare/micro-common");
class MicroRedisStartup extends micro_1.MicroStartup {
    constructor(options = {}) {
        super();
        this.options = options;
        _MicroRedisStartup_instances.add(this);
        _MicroRedisStartup_handlers.set(this, []);
    }
    listen() {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.close();
            const opt = Object.assign({}, this.options);
            if (!("url" in opt)) {
                const port = (0, core_1.getIparePort)(6379);
                opt.url = `redis://localhost:${port}`;
            }
            // eslint-disable-next-line @typescript-eslint/no-var-requires
            const redisPkg = require("redis");
            const pub = redisPkg.createClient(opt);
            const sub = redisPkg.createClient(opt);
            this.pub = pub;
            this.sub = sub;
            yield Promise.all([pub.connect(), sub.connect()]);
            __classPrivateFieldGet(this, _MicroRedisStartup_handlers, "f").forEach((item) => {
                __classPrivateFieldGet(this, _MicroRedisStartup_instances, "m", _MicroRedisStartup_pattern).call(this, item.pattern, item.handler);
            });
            this.logger.info(`Server started, listening url: ${opt.url}`);
            return {
                sub: this.sub,
                pub: this.pub,
            };
        });
    }
    pattern(pattern, handler) {
        this.logger.debug(`Add pattern: ${pattern}`);
        __classPrivateFieldGet(this, _MicroRedisStartup_handlers, "f").push({ pattern, handler });
        return __classPrivateFieldGet(this, _MicroRedisStartup_instances, "m", _MicroRedisStartup_pattern).call(this, pattern, handler);
    }
    patterns(...patterns) {
        patterns.forEach((item) => {
            this.pattern(item.pattern, item.handler);
        });
        return this;
    }
    close() {
        return __awaiter(this, void 0, void 0, function* () {
            function disconnect(redis) {
                return __awaiter(this, void 0, void 0, function* () {
                    if ((redis === null || redis === void 0 ? void 0 : redis.isReady) && redis.isOpen) {
                        yield redis.quit();
                    }
                });
            }
            yield disconnect(this.pub);
            this.pub = undefined;
            yield disconnect(this.sub);
            this.sub = undefined;
            this.logger.info("Server shutdown success");
        });
    }
}
exports.MicroRedisStartup = MicroRedisStartup;
_MicroRedisStartup_handlers = new WeakMap(), _MicroRedisStartup_instances = new WeakSet(), _MicroRedisStartup_pattern = function _MicroRedisStartup_pattern(pattern, handler) {
    if (!this.sub)
        return this;
    this.sub.subscribe(pattern, (buffer) => __awaiter(this, void 0, void 0, function* () {
        this.handleMessage((0, micro_common_1.parseJsonBuffer)(buffer), ({ result, req }) => __awaiter(this, void 0, void 0, function* () {
            var _a;
            yield ((_a = this.pub) === null || _a === void 0 ? void 0 : _a.publish(pattern + "." + req.id, JSON.stringify(result)));
        }), handler);
    }), true);
    return this;
};
